==> react-app-logic-11.11.0.tgz: package/src/gov/forms/useProposalDepositForm.ts <==
import { useCallback, useEffect, useMemo, useState } from 'react';
import BigNumber from 'bignumber.js';
import { useAsyncProcess, useAsyncStatuses, useAsyncValue } from '@onomy/react-utils';
import { DetailedFungibleAssetBalance } from '@onomy/hub';
import {
  useAssetBalance,
  useDetailedBalance,
  useGasForm,
  useOnomyHub,
  useFixedAmountInput,
  useProposal,
  useProposalMinDeposit,
} from '@onomy/react-hub';

import { useOnomyApp } from '../../OnomyApp';

type ProposalDepositResult = {
  proposalId: string;
  balanceBefore: DetailedFungibleAssetBalance;
  balanceAfter: DetailedFungibleAssetBalance;
  amount: DetailedFungibleAssetBalance;
  gasAmount: DetailedFungibleAssetBalance;
};

export function useProposalDepositForm({
  proposalId,
  autoSetNeededAmount: autoSetNeededAmountProp = false,
}: {
  proposalId: string;
  autoSetNeededAmount?: boolean;
}) {
  const { hub } = useOnomyHub();
  const { onomyNomAsset } = useOnomyApp();

  const [attempt, txStatus, txReset] = useAsyncProcess();
  const [txDetails, setTxDetails] = useState<ProposalDepositResult | null>(null);

  const [proposal, proposalStatus] = useProposal(proposalId);
  const { asset = onomyNomAsset } = proposal ?? {};
  const [balance, balanceStatus] = useAssetBalance(asset);

  const [amountFixed, setAmountFixed] = useState('');
  const [autoSetNeededAmount, setAutoSetNeededAmount] = useState(autoSetNeededAmountProp);
  const toggleAutoSetNeededAmount = useCallback(() => setAutoSetNeededAmount(v => !v), []);

  const [amount, amountStatus] = useDetailedBalance(asset, amountFixed);
  const [minDepositAmount, minDepositAmountStatus] = useProposalMinDeposit();

  // TODO: support multiple deposit denoms?
  const depositNeededFixed =
    minDepositAmount.amount - (proposal?.totalDeposits[0]?.amount ?? BigInt(0));
  const [depositNeeded, depositNeededStatus] = useDetailedBalance(
    asset,
    depositNeededFixed.toString(10)
  );

  useEffect(() => {
    if (autoSetNeededAmount) setAmountFixed(depositNeededFixed.toString(10));
  }, [depositNeededFixed, autoSetNeededAmount]);

  const [amountInput, setAmountInput] = useFixedAmountInput({
    value: amountFixed,
    onChangeValue: setAmountFixed,
  });

  const inputParams = useMemo(() => {
    return {
      proposalId,
      amountFixed: new BigNumber(amountFixed),
    };
  }, [proposalId, amountFixed]);

  const gas = useGasForm(
    asset,
    asset,
    async () => {
      if (!hub) return '0';
      const res = await hub.chains.onomy.client.feesForDepositToProposal(inputParams);
      const [denomAmount] = res.amount;
      return denomAmount.amount;
    },
    [hub, inputParams]
  );

  const loadingStatus = useAsyncStatuses([
    proposalStatus,
    balanceStatus,
    amountStatus,
    minDepositAmountStatus,
    depositNeededStatus,
  ]);

  const hasEnoughBalance = useMemo(() => {
    let total = amount.amount;

    if (amount.asset.id === gas.gasAmount.asset.id) total += gas.gasAmount.amount;
    return balance.amount >= total;
  }, [
    amount.amount,
    amount.asset.id,
    gas.gasAmount.asset.id,
    gas.gasAmount.amount,
    balance.amount,
  ]);

  const formErrors = useMemo(() => {
    const errors: {
      gas?: string;
      amountInput?: string;
      proposalId?: string;
    } = {};

    if (!inputParams.proposalId) {
      errors.proposalId = 'Proposal not set';
    }

    if (!inputParams.amountFixed) {
      errors.amountInput = 'Amount is required';
    } else {
      const amountFixedValue = new BigNumber(inputParams.amountFixed);
      if (amountFixedValue.lte(0) || amountFixedValue.isNaN()) {
        errors.amountInput = 'Amount must be greater than 0';
      }
    }

    if (!gas.hasEnoughGasBalance) errors.gas = `Insuffcient ${gas.gasAsset.meta.symbol}`;
    if (!hasEnoughBalance) errors.amountInput = `Insuffcient ${balance.asset.meta.symbol}`;

    return Object.keys(errors).length > 0 ? errors : null;
  }, [
    inputParams.proposalId,
    inputParams.amountFixed,
    gas.hasEnoughGasBalance,
    gas.gasAsset.meta.symbol,
    hasEnoughBalance,
    balance.asset.meta.symbol,
  ]);

  const canSubmit = useMemo(() => {
    return !formErrors && !loadingStatus.pending && !loadingStatus.error && !!gas.gasPrice;
  }, [formErrors, gas.gasPrice, loadingStatus]);

  const reset = useCallback(() => {
    setAmountFixed('');
    txReset();
    setTxDetails(null);
  }, [txReset]);

  const submit = useCallback(() => {
    return attempt(async () => {
      if (!hub || !canSubmit) return;

      await hub.chains.onomy.client.depositToProposal({
        ...inputParams,
        feeMultiplier: parseInt(gas.gasPrice, 10),
      });

      // Force update of proposal data
      await hub.proposals.forceGet('onomy', inputParams.proposalId);
      const balanceAfter = await hub.balances.forceGet(balance.asset);

      const result = {
        proposalId: inputParams.proposalId,
        amount,
        gasAmount: gas.gasAmount,
        balanceBefore: balance,
        balanceAfter,
      };

      setTxDetails(result);
    });
  }, [attempt, hub, canSubmit, amount, balance, gas.gasAmount, gas.gasPrice, inputParams]);

  return {
    loadingStatus,
    txStatus,
    txDetails,

    proposal,
    asset,
    balance,
    amount,
    minDepositAmount,
    depositNeeded,

    amountInput,
    autoSetNeededAmount,
    gas,

    setAmountInput,
    setAutoSetNeededAmount,
    toggleAutoSetNeededAmount,

    hasEnoughBalance,
    formErrors,
    canSubmit,

    reset,
    submit,
  };
}

==> react-app-logic-12.1.0.tgz: package/src/gov/forms/useProposalDepositForm.ts <==
import { useCallback, useEffect, useMemo, useState } from 'react';
import BigNumber from 'bignumber.js';
import { useAsyncProcess, useAsyncStatuses, useAsyncValue } from '@onomy/react-utils';
import { DetailedFungibleAssetBalance } from '@onomy/hub';
import {
  useAssetBalance,
  useDetailedBalance,
  useGasForm,
  useOnomyHub,
  useFixedAmountInput,
  useProposal,
  useProposalMinDeposit,
} from '@onomy/react-hub';

import { useOnomyApp } from '../../wallet/providers/OnomyApp';

type ProposalDepositResult = {
  proposalId: string;
  balanceBefore: DetailedFungibleAssetBalance;
  balanceAfter: DetailedFungibleAssetBalance;
  amount: DetailedFungibleAssetBalance;
  gasAmount: DetailedFungibleAssetBalance;
};

export function useProposalDepositForm({
  proposalId,
  autoSetNeededAmount: autoSetNeededAmountProp = false,
}: {
  proposalId: string;
  autoSetNeededAmount?: boolean;
}) {
  const { hub } = useOnomyHub();
  const { onomyNomAsset } = useOnomyApp();

  const [attempt, txStatus, txReset] = useAsyncProcess();
  const [txDetails, setTxDetails] = useState<ProposalDepositResult | null>(null);

  const [proposal, proposalStatus] = useProposal(proposalId);
  const { asset = onomyNomAsset } = proposal ?? {};
  const [balance, balanceStatus] = useAssetBalance(asset);

  const [amountFixed, setAmountFixed] = useState('');
  const [autoSetNeededAmount, setAutoSetNeededAmount] = useState(autoSetNeededAmountProp);
  const toggleAutoSetNeededAmount = useCallback(() => setAutoSetNeededAmount(v => !v), []);

  const [amount, amountStatus] = useDetailedBalance(asset, amountFixed);
  const [minDepositAmount, minDepositAmountStatus] = useProposalMinDeposit();

  // TODO: support multiple deposit denoms?
  const depositNeededFixed =
    minDepositAmount.amount - (proposal?.totalDeposits[0]?.amount ?? BigInt(0));
  const [depositNeeded, depositNeededStatus] = useDetailedBalance(
    asset,
    depositNeededFixed.toString(10)
  );

  useEffect(() => {
    if (autoSetNeededAmount) setAmountFixed(depositNeededFixed.toString(10));
  }, [depositNeededFixed, autoSetNeededAmount]);

  const [amountInput, setAmountInput] = useFixedAmountInput({
    value: amountFixed,
    onChangeValue: setAmountFixed,
  });

  const inputParams = useMemo(() => {
    return {
      proposalId,
      amountFixed: new BigNumber(amountFixed),
    };
  }, [proposalId, amountFixed]);

  const gas = useGasForm(
    asset,
    asset,
    async () => {
      if (!hub) return '0';
      const res = await hub.chains.onomy.client.feesForDepositToProposal(inputParams);
      const [denomAmount] = res.amount;
      return denomAmount.amount;
    },
    [hub, inputParams]
  );

  const loadingStatus = useAsyncStatuses([
    proposalStatus,
    balanceStatus,
    amountStatus,
    minDepositAmountStatus,
    depositNeededStatus,
  ]);

  const hasEnoughBalance = useMemo(() => {
    let total = amount.amount;

    if (amount.asset.id === gas.gasAmount.asset.id) total += gas.gasAmount.amount;
    return balance.amount >= total;
  }, [
    amount.amount,
    amount.asset.id,
    gas.gasAmount.asset.id,
    gas.gasAmount.amount,
    balance.amount,
  ]);

  const formErrors = useMemo(() => {
    const errors: {
      gas?: string;
      amountInput?: string;
      proposalId?: string;
    } = {};

    if (!inputParams.proposalId) {
      errors.proposalId = 'Proposal not set';
    }

    if (!inputParams.amountFixed) {
      errors.amountInput = 'Amount is required';
    } else {
      const amountFixedValue = new BigNumber(inputParams.amountFixed);
      if (amountFixedValue.lte(0) || amountFixedValue.isNaN()) {
        errors.amountInput = 'Amount must be greater than 0';
      }
    }

    if (!gas.hasEnoughGasBalance) errors.gas = `Insuffcient ${gas.gasAsset.meta.symbol}`;
    if (!hasEnoughBalance) errors.amountInput = `Insuffcient ${balance.asset.meta.symbol}`;

    return Object.keys(errors).length > 0 ? errors : null;
  }, [
    inputParams.proposalId,
    inputParams.amountFixed,
    gas.hasEnoughGasBalance,
    gas.gasAsset.meta.symbol,
    hasEnoughBalance,
    balance.asset.meta.symbol,
  ]);

  const canSubmit = useMemo(() => {
    return !formErrors && !loadingStatus.pending && !loadingStatus.error && !!gas.gasPrice;
  }, [formErrors, gas.gasPrice, loadingStatus]);

  const reset = useCallback(() => {
    setAmountFixed('');
    txReset();
    setTxDetails(null);
  }, [txReset]);

  const submit = useCallback(() => {
    return attempt(async () => {
      if (!hub || !canSubmit) return;

      await hub.chains.onomy.client.depositToProposal({
        ...inputParams,
        feeMultiplier: parseInt(gas.gasPrice, 10),
      });

      // Force update of proposal data
      await hub.proposals.forceGet('onomy', inputParams.proposalId);
      const balanceAfter = await hub.balances.forceGet(balance.asset);

      const result = {
        proposalId: inputParams.proposalId,
        amount,
        gasAmount: gas.gasAmount,
        balanceBefore: balance,
        balanceAfter,
      };

      setTxDetails(result);
    });
  }, [attempt, hub, canSubmit, amount, balance, gas.gasAmount, gas.gasPrice, inputParams]);

  return {
    loadingStatus,
    txStatus,
    txDetails,

    proposal,
    asset,
    balance,
    amount,
    minDepositAmount,
    depositNeeded,

    amountInput,
    autoSetNeededAmount,
    gas,

    setAmountInput,
    setAutoSetNeededAmount,
    toggleAutoSetNeededAmount,

    hasEnoughBalance,
    formErrors,
    canSubmit,

    reset,
    submit,
  };
}
